@@ -22,6 +22,8 @@ import tempfile, types
 import traceback
 import errno
 import time
+import getpass
+import subprocess
 
 try:
     import util
@@ -152,6 +154,7 @@ class MozillaDelegate(userprofile.SourceDelegate):
         self.committed_bookmarks = mozilla_bookmarks.BookmarkFolder("Null", None)
         self.committed_mandatory_bookmarks = mozilla_bookmarks.BookmarkFolder("Null", None)
         self.ini_file = None
+        self.SORTPRIORITY = 50
 
     def get_full_path(self, path):
         return os.path.join(self.home_dir, path)        
@@ -212,8 +215,7 @@ class MozillaDelegate(userprofile.SourceDelegate):
         # INI File
         #
         if self.is_ini_file(rel_path):
-            dprint(LOG_CHANGE, "%s ini file: %s",
-                   dirmonitor.event_to_string(change.event), rel_path)
+            dprint(LOG_CHANGE, "%s ini file: %s", dirmonitor.event_to_string(change.event), rel_path)
             self.load_profiles_ini()
 
         #
@@ -221,8 +223,7 @@ class MozillaDelegate(userprofile.SourceDelegate):
         #
         profile = self.is_profile_file(rel_path)
         if profile:
-            dprint(LOG_CHANGE, "%s profile file: %s",
-                   dirmonitor.event_to_string(change.event), rel_path)
+            dprint(LOG_CHANGE, "%s profile file: %s", dirmonitor.event_to_string(change.event), rel_path)
 
             profile_file = profile.add_file(rel_path)
             profile_file_type = profile_file.get_type()
@@ -392,9 +393,20 @@ class MozillaDelegate(userprofile.SourceDelegate):
         dprint(LOG_APPLY, "apply: ini_files=%s pref_files=%s bookmark_files=%s other_files=%s",
                ini_files, pref_files, bookmark_files, other_files)
 
+        def which_firefox ():
+            path = os.environ['PATH']
+            paths = path.split(os.pathsep)
+            for p in paths:
+                f = os.path.join (p, "firefox")
+                if os.path.exists (f):
+                    return f
+            else:
+                return None
+
         # Profiles.ini file must be done first, if the target does not
         # exist then extract it from the profile.
         # Parse the profiles.ini file to learn the target profiles
+        # If no profile exists in homedir or .zip, create new profile
         self.load_profiles_ini()
         if not self.ini_file.is_valid():
             dprint(LOG_APPLY, "apply: no valid ini file, extracting %s", profiles_ini_rel_path)
@@ -402,11 +414,17 @@ class MozillaDelegate(userprofile.SourceDelegate):
                 self.source.storage.extract(profiles_ini_rel_path, self.home_dir, True)
                 self.load_profiles_ini()
             else:
-                dprint(LOG_APPLY, "apply: but there isn't an ini file in the profile!")
+                dprint(LOG_APPLY, "apply: but there isn't an ini file in the profile.  Creating new profile.")
+                firefoxpath = which_firefox ()
+                if firefoxpath:
+                    subprocess.call ([firefoxpath, "-createProfile", getpass.getuser ()])
+                    self.load_profiles_ini()
+                else:
+                    print "Firefox not found in PATH.  Unable to create new profile."
 
         # --------------------
         if sabayon_pref_rel_path in pref_files:
-            dprint(LOG_APPLY, "extracting %s" % sabayon_pref_rel_path)
+            dprint(LOG_APPLY, "extracting %s", sabayon_pref_rel_path)
             self.source.storage.extract(sabayon_pref_rel_path, self.home_dir, True)
             apply_pref = JavascriptPrefsFile(self.home_dir, sabayon_pref_rel_path)
             apply_pref.read()
@@ -414,7 +432,7 @@ class MozillaDelegate(userprofile.SourceDelegate):
             apply_pref = None
 
         if sabayon_mandatory_pref_rel_path in pref_files:
-            dprint(LOG_APPLY, "extracting %s" % sabayon_mandatory_pref_rel_path)
+            dprint(LOG_APPLY, "extracting %s", sabayon_mandatory_pref_rel_path)
             self.source.storage.extract(sabayon_mandatory_pref_rel_path, self.home_dir, True)
             mandatory_apply_pref = JavascriptPrefsFile(self.home_dir, sabayon_mandatory_pref_rel_path)
             mandatory_apply_pref.read()
@@ -424,7 +442,7 @@ class MozillaDelegate(userprofile.SourceDelegate):
         # --------------------
 
         if sabayon_bookmark_rel_path in bookmark_files:
-            dprint(LOG_APPLY, "extracting %s" % sabayon_bookmark_rel_path)
+            dprint(LOG_APPLY, "extracting %s", sabayon_bookmark_rel_path)
             self.source.storage.extract(sabayon_bookmark_rel_path, self.home_dir, True)
             apply_bookmark = BookmarksFile(self.home_dir, sabayon_bookmark_rel_path)
             apply_bookmark.read()
@@ -432,7 +450,7 @@ class MozillaDelegate(userprofile.SourceDelegate):
             apply_bookmark = None
 
         if sabayon_mandatory_bookmark_rel_path in bookmark_files:
-            dprint(LOG_APPLY, "extracting %s" % sabayon_mandatory_bookmark_rel_path)
+            dprint(LOG_APPLY, "extracting %s", sabayon_mandatory_bookmark_rel_path)
             self.source.storage.extract(sabayon_mandatory_bookmark_rel_path, self.home_dir, True)
             mandatory_apply_bookmark = BookmarksFile(self.home_dir, sabayon_mandatory_bookmark_rel_path)
             mandatory_apply_bookmark.read()
@@ -714,8 +732,7 @@ class JavascriptPrefsFile(FirefoxProfileFile):
                 type  = match.group(1)
                 key   = match.group(2)
                 value = match.group(3)
-                dprint(LOG_PARSE, "(%d:%d) key='%s' value='%s'" %
-                       (match.start(), match.end(), key, value))
+                dprint(LOG_PARSE, "(%d:%d) key='%s' value='%s'", match.start(), match.end(), key, value)
                 self.prefs[key] = JavascriptPreference(type, key, value)
                 start = match.end()
             else:
@@ -731,7 +748,7 @@ class JavascriptPrefsFile(FirefoxProfileFile):
         keys = self.prefs.keys()
         keys.sort()
         for key in keys:
-            dprint(LOG_, "%s=%s" % (key, self.prefs[key]))
+            dprint(LOG_VERBOSE, "%s=%s", key, self.prefs[key])
 
 
 # ------ Class FirefoxProfile ------
@@ -833,8 +850,7 @@ class FirefoxProfilesIni:
 
 
     def read(self):
-        dprint(LOG_OPERATION, "FirefoxProfilesIni.read() path = %s",
-               self.get_full_path(self.rel_path))
+        dprint(LOG_OPERATION, "FirefoxProfilesIni.read() path = %s", self.get_full_path(self.rel_path))
         self.profiles = {}
 
         try:
@@ -1121,11 +1137,11 @@ def get_type_from_path(rel_path):
 
 def cat_file(path):
     if os.path.isfile(path):
-        dprint(LOG_FILE_CONTENTS, "==== %s ====" % path)
+        dprint(LOG_FILE_CONTENTS, "==== %s ====", path)
         for line in open(path):
             dprint(LOG_FILE_CONTENTS, line.rstrip())
     else:
-        dprint(LOG_FILE_CONTENTS, "WARNING, does not exist ==== %s ====" % path)
+        dprint(LOG_FILE_CONTENTS, "WARNING, does not exist ==== %s ====", path)
 
 
 
